﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports System.Globalization
Imports System.Windows.Controls.Primitives


Public Class ColorPicker
	Inherits Control
	Shared Sub New()
		DefaultStyleKeyProperty.OverrideMetadata(GetType(ColorPicker), New FrameworkPropertyMetadata(GetType(ColorPicker)))
	End Sub

	Public Overrides Sub OnApplyTemplate()
		MyBase.OnApplyTemplate()

		popup = TryCast(GetTemplateChild("popup"), Popup)
		alpha = TryCast(GetTemplateChild("alpha"), Slider)
		red = TryCast(GetTemplateChild("red"), Slider)
		green = TryCast(GetTemplateChild("green"), Slider)
		blue = TryCast(GetTemplateChild("blue"), Slider)

		If alpha IsNot Nothing Then
			AddHandler alpha.ValueChanged, AddressOf OnAlphaChanged
		End If
		If red IsNot Nothing Then
			AddHandler red.ValueChanged, AddressOf OnRedChanged
		End If
		If green IsNot Nothing Then
			AddHandler green.ValueChanged, AddressOf OnGreenChanged
		End If
		If blue IsNot Nothing Then
			AddHandler blue.ValueChanged, AddressOf OnBlueChanged
		End If
	End Sub

	Private Sub OnAlphaChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
		Dim color As Color = SelectedColor
		color.A = CByte(e.NewValue)
		SelectedColor = color
		RaiseSelectedColorChanged()
	End Sub

	Private Sub OnRedChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
		Dim color As Color = SelectedColor
		color.R = CByte(e.NewValue)
		SelectedColor = color
		RaiseSelectedColorChanged()
	End Sub

	Private Sub OnGreenChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
		Dim color As Color = SelectedColor
		color.G = CByte(e.NewValue)
		SelectedColor = color
		RaiseSelectedColorChanged()
	End Sub

	Private Sub OnBlueChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
		Dim color As Color = SelectedColor
		color.B = CByte(e.NewValue)
		SelectedColor = color
		RaiseSelectedColorChanged()
	End Sub

	Protected Overrides Sub OnMouseUp(ByVal e As System.Windows.Input.MouseButtonEventArgs)
		MyBase.OnMouseUp(e)

		If Not IsEnabled Then
			Return
		End If

		alpha.Value = SelectedColor.A
		red.Value = SelectedColor.R
		green.Value = SelectedColor.G
		blue.Value = SelectedColor.B
		popup.IsOpen = True
	End Sub

	Private Sub RaiseSelectedColorChanged()
		Dim args As New RoutedEventArgs(ColorPicker.SelectedColorChangedEvent)
		OnSelectedColorChanged(args)
	End Sub

	Protected Overridable Sub OnSelectedColorChanged(ByVal e As RoutedEventArgs)
		[RaiseEvent](e)
	End Sub


	Public Property SelectedColor() As Color
		Get
			Return DirectCast(GetValue(SelectedColorProperty), Color)
		End Get
		Set(ByVal value As Color)
			SetValue(SelectedColorProperty, value)
		End Set
	End Property

	Public Shared ReadOnly SelectedColorProperty As DependencyProperty = _
	 DependencyProperty.Register("SelectedColor", GetType(Color), GetType(ColorPicker))


	' Create a custom routed event by first registering a RoutedEventID 
	' This event uses the bubbling routing strategy 
	Public Shared ReadOnly SelectedColorChangedEvent As RoutedEvent = _
	 EventManager.RegisterRoutedEvent("SelectedColorChanged", RoutingStrategy.Bubble, GetType(RoutedEventHandler), GetType(ColorPicker))

	Public Custom Event SelectedColorChanged As RoutedEventHandler
		AddHandler(ByVal value As RoutedEventHandler)
			[AddHandler](SelectedColorChangedEvent, value)
		End AddHandler
		RemoveHandler(ByVal value As RoutedEventHandler)
			[RemoveHandler](SelectedColorChangedEvent, value)
		End RemoveHandler
		RaiseEvent(ByVal sender As System.Object, ByVal e As System.EventArgs)
			RaiseSelectedColorChanged()
		End RaiseEvent
	End Event


	Private popup As Popup
	Private alpha As Slider
	Private red As Slider
	Private green As Slider
	Private blue As Slider
End Class


Public Class ColorConverter
	Implements IValueConverter
	Public Function Convert(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As CultureInfo) As Object Implements IValueConverter.Convert
		If TypeOf value Is Color Then
			Return New SolidColorBrush(DirectCast(value, Color))
		End If

		Return value
	End Function

	Public Function ConvertBack(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As CultureInfo) As Object Implements IValueConverter.ConvertBack
		Throw New NotImplementedException()
	End Function
End Class

Public Class VisibilityConverter
	Implements IValueConverter
	Public Function Convert(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As CultureInfo) As Object Implements IValueConverter.Convert
		If TypeOf value Is Boolean Then
			Dim bvalue As Boolean = CBool(value)
			If parameter IsNot Nothing Then
				If parameter.ToString() = "!" Then
					bvalue = Not bvalue
				End If
			End If

			If bvalue Then
				Return Visibility.Visible
			Else
				Return Visibility.Collapsed
			End If
		End If

		Return value
	End Function

	Public Function ConvertBack(ByVal value As Object, ByVal targetType As Type, ByVal parameter As Object, ByVal culture As CultureInfo) As Object Implements IValueConverter.ConvertBack
		Throw New NotImplementedException()
	End Function
End Class