﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'


''' <summary>
''' Interaction logic for MainWindow.xaml
''' </summary>
Partial Public Class MainWindow
	Inherits Window

	Public Sub New()

		InitializeComponent()

		' Create effects
		glassEffect = New GlassEffect()
		aeroEffect = New AeroEffect()

		' Preview shapes
		Dim i As Integer = 0
		Const size As Double = 75
		Const margin As Double = 10
		Const perLine As Integer = 7
		For Each shape As Shape In Shape.Shapes
			If shape.IsArrowhead Then
				Continue For
			End If
			' BPMN shapes don't add variety
			If shape.Id.StartsWith("bpmn", StringComparison.InvariantCultureIgnoreCase) Then
				Continue For
			End If

			Dim node As ShapeNode = diagram.Factory.CreateShapeNode((i Mod perLine) * (size + margin) + 4, (i \ perLine) * (size + margin) + 4, size, size)
			node.Shape = shape
			i += 1
		Next
		diagram.ResizeToFitItems(4, False)

		' Presets
		defaultPreset = New Preset()
		defaultPreset.Name = "(Default)"
		defaultPreset.UseGlassEffect = False
		defaultPreset.GlassEffectType = GlassEffectType.Type1
		defaultPreset.UsePenAsGlow = False
		defaultPreset.GlowColor = Colors.White
		defaultPreset.ReflectionColor = Colors.White
		defaultPreset.UseAeroEffect = False
		defaultPreset.Opacity = 0.4
		defaultPreset.InnerOutlineColor = Colors.White
		defaultPreset.ShadeColor = Colors.Black
		defaultPreset.UseEmbossEffect = False
		defaultPreset.UseSmoothShadowEffect = False
		defaultPreset.DiagramBackground = Colors.White
		defaultPreset.ShapeBorder = Colors.DarkSlateGray
		defaultPreset.ShapeBackground1 = Colors.LightSteelBlue
		defaultPreset.UseShapeBackground2 = False
		defaultPreset.ShapeBackground2 = Colors.White
		ApplyPreset(defaultPreset)

		Dim preset As New Preset()
		preset.Name = "Glass (Default Colors)"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type1
		preset.UsePenAsGlow = False
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = False
		preset.Opacity = 0.4
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Colors.White
		preset.ShapeBorder = Colors.DarkSlateGray
		preset.ShapeBackground1 = Colors.LightSteelBlue
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Glass & Aero (Orange Background)"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type2
		preset.UsePenAsGlow = False
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = True
		preset.Opacity = 0.2
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Colors.Orange
		preset.ShapeBorder = Colors.DarkSlateGray
		preset.ShapeBackground1 = Colors.LightSteelBlue
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Metalic"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type2
		preset.UsePenAsGlow = True
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = True
		preset.Opacity = 0.5
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Color.FromArgb(&HFF, &HC0, &HC0, &HC0)
		preset.ShapeBorder = Colors.DarkSlateGray
		preset.ShapeBackground1 = Colors.LightSteelBlue
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Neon Glow"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type2
		preset.UsePenAsGlow = True
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = True
		preset.Opacity = 0
		preset.InnerOutlineColor = Colors.Black
		preset.ShadeColor = Color.FromArgb(&HFF, &H80, &HFF, &HFF)
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Colors.Black
		preset.ShapeBorder = Color.FromArgb(&HFF, &H80, &HFF, &HFF)
		preset.ShapeBackground1 = Colors.Black
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Gray glass"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type3
		preset.UsePenAsGlow = False
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = True
		preset.Opacity = 0.4
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Colors.Gray
		preset.ShapeBorder = Colors.Black
		preset.ShapeBackground1 = Colors.LightCyan
		preset.UseShapeBackground2 = True
		preset.ShapeBackground2 = Color.FromArgb(&HFF, &HA8, &HBF, &HBF)
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Brown"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type1
		preset.UsePenAsGlow = False
		preset.GlowColor = Colors.White
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = False
		preset.Opacity = 0.4
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = True
		preset.DiagramBackground = Colors.PaleGoldenrod
		preset.ShapeBorder = Colors.Black
		preset.ShapeBackground1 = Color.FromArgb(&H80, &H90, &H30, &H20)
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		preset = New Preset()
		preset.Name = "Bright Red"
		preset.UseGlassEffect = True
		preset.GlassEffectType = GlassEffectType.Type4
		preset.UsePenAsGlow = False
		preset.GlowColor = Colors.Black
		preset.ReflectionColor = Colors.White
		preset.UseAeroEffect = False
		preset.Opacity = 0.4
		preset.InnerOutlineColor = Colors.White
		preset.ShadeColor = Colors.Black
		preset.UseEmbossEffect = False
		preset.UseSmoothShadowEffect = False
		preset.DiagramBackground = Colors.White
		preset.ShapeBorder = Color.FromArgb(&HFF, &H8E, &H8F, &H8F)
		preset.ShapeBackground1 = Colors.Red
		preset.UseShapeBackground2 = False
		preset.ShapeBackground2 = Colors.White
		AddToList(preset)

		AddHandler backgroundColor.SelectedColorChanged, AddressOf backgroundColor_SelectedColorChanged
		AddHandler shapesBorderColor.SelectedColorChanged, AddressOf shapesBorderColor_SelectedColorChanged
		AddHandler shapesBackgroundColor.SelectedColorChanged, AddressOf shapesBackgroundColor_SelectedColorChanged
		AddHandler shapesBackground2Color.SelectedColorChanged, AddressOf shapesBackground2Color_SelectedColorChanged
		AddHandler glowColor.SelectedColorChanged, AddressOf glowColor_SelectedColorChanged
		AddHandler reflectionColor.SelectedColorChanged, AddressOf reflectionColor_SelectedColorChanged
		AddHandler innerOutlineColor.SelectedColorChanged, AddressOf innerOutlineColor_SelectedColorChanged
		AddHandler shadeColor.SelectedColorChanged, AddressOf shadeColor_SelectedColorChanged
	End Sub

	Private Sub AddToList(ByVal preset As Preset)
		presetsList.Items.Add(preset)
	End Sub

	Private Sub presetsList_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
		ApplyPreset(TryCast(presetsList.SelectedItem, Preset))
	End Sub

	Private Sub glassEffectCheck_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		If glassEffectCheck.IsChecked = True Then
			If Not diagram.NodeEffects.Contains(glassEffect) Then
				diagram.NodeEffects.Add(glassEffect)
			End If
		Else
			If diagram.NodeEffects.Contains(glassEffect) Then
				diagram.NodeEffects.Remove(glassEffect)
			End If
		End If
	End Sub

	Private Sub glassTypeCombo_SelectionChanged(ByVal sender As Object, ByVal e As SelectionChangedEventArgs)
		glassEffect.Type = DirectCast(glassTypeCombo.SelectedIndex, GlassEffectType)
	End Sub

	Private Sub usePenAsGlowCheck_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		glassEffect.UsePenAsGlow = usePenAsGlowCheck.IsChecked = True
	End Sub

	Private Sub glowColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		glassEffect.GlowColor = glowColor.SelectedColor
	End Sub

	Private Sub reflectionColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		glassEffect.ReflectionColor = reflectionColor.SelectedColor
	End Sub

	Private Sub aeroEffectCheck_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		If aeroEffectCheck.IsChecked = True Then
			If Not diagram.NodeEffects.Contains(aeroEffect) Then
				diagram.NodeEffects.Add(aeroEffect)
			End If
		Else
			If diagram.NodeEffects.Contains(aeroEffect) Then
				diagram.NodeEffects.Remove(aeroEffect)
			End If
		End If
	End Sub

	Private Sub aeroOpacity_ValueChanged(ByVal sender As Object, ByVal e As RoutedPropertyChangedEventArgs(Of Double))
		aeroEffect.Opacity = aeroOpacity.Value
	End Sub

	Private Sub innerOutlineColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		aeroEffect.InnerOutlineColor = innerOutlineColor.SelectedColor
	End Sub

	Private Sub shadeColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		aeroEffect.ShadeColor = shadeColor.SelectedColor
	End Sub

	Private Sub backgroundColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.BackBrush = New SolidColorBrush(backgroundColor.SelectedColor)
	End Sub

	Private Sub shapesBorderColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		UpdateShapeStyle()
	End Sub

	Private Sub shapesBackgroundColor_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		UpdateShapeStyle()
	End Sub

	Private Sub gradientCheck_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		UpdateShapeStyle()
	End Sub

	Private Sub shapesBackground2Color_SelectedColorChanged(ByVal sender As Object, ByVal e As RoutedEventArgs)
		UpdateShapeStyle()
	End Sub

	Private Sub UpdateShapeStyle()
		Dim stroke = New SolidColorBrush(shapesBorderColor.SelectedColor)

		Dim brush As Brush = Nothing
		If Not gradientCheck.IsChecked = True Then
			brush = New SolidColorBrush(shapesBackgroundColor.SelectedColor)
		Else
			brush = New LinearGradientBrush(shapesBackgroundColor.SelectedColor, shapesBackground2Color.SelectedColor, 90)
		End If

		Dim shapeNodeStyle As New Style()
		shapeNodeStyle.Setters.Add(New Setter(ShapeNode.BrushProperty, brush))
		shapeNodeStyle.Setters.Add(New Setter(ShapeNode.StrokeProperty, stroke))
		diagram.ShapeNodeStyle = shapeNodeStyle
	End Sub

	Private Sub ApplyPreset(ByVal preset As Preset)
		glassEffectCheck.IsChecked = preset.UseGlassEffect
		glassTypeCombo.SelectedIndex = CInt(preset.GlassEffectType)
		usePenAsGlowCheck.IsChecked = preset.UsePenAsGlow
		glowColor.SelectedColor = preset.GlowColor
		reflectionColor.SelectedColor = preset.ReflectionColor

		aeroEffectCheck.IsChecked = preset.UseAeroEffect
		aeroOpacity.Value = preset.Opacity
		shadeColor.SelectedColor = preset.ShadeColor
		innerOutlineColor.SelectedColor = preset.InnerOutlineColor

		backgroundColor.SelectedColor = preset.DiagramBackground
		shapesBorderColor.SelectedColor = preset.ShapeBorder
		shapesBackgroundColor.SelectedColor = preset.ShapeBackground1
		gradientCheck.IsChecked = preset.UseShapeBackground2
		shapesBackground2Color.SelectedColor = preset.ShapeBackground2

		glassEffectCheck_Click(Nothing, Nothing)
		glassTypeCombo_SelectionChanged(Nothing, Nothing)
		usePenAsGlowCheck_Click(Nothing, Nothing)
		glowColor_SelectedColorChanged(Nothing, Nothing)
		reflectionColor_SelectedColorChanged(Nothing, Nothing)
		aeroEffectCheck_Click(Nothing, Nothing)
		aeroOpacity_ValueChanged(Nothing, Nothing)
		shadeColor_SelectedColorChanged(Nothing, Nothing)
		innerOutlineColor_SelectedColorChanged(Nothing, Nothing)
		backgroundColor_SelectedColorChanged(Nothing, Nothing)
		UpdateShapeStyle()
	End Sub


	Private glassEffect As GlassEffect
	Private aeroEffect As AeroEffect
	Private defaultPreset As Preset

	Private Class Preset
		Public Property PublicName() As String
			Get
				Return Name
			End Get
			Set(ByVal value As String)
				Name = value
			End Set
		End Property

		Public Name As String

		' Glass
		Public UseGlassEffect As Boolean
		Public GlassEffectType As GlassEffectType
		Public UsePenAsGlow As Boolean
		Public GlowColor As Color
		Public ReflectionColor As Color

		' Aero
		Public UseAeroEffect As Boolean
		Public Opacity As Double
		Public InnerOutlineColor As Color
		Public ShadeColor As Color

		' Emboss
		Public UseEmbossEffect As Boolean

		' Shadow
		Public UseSmoothShadowEffect As Boolean

		' Misc
		Public DiagramBackground As Color
		Public ShapeBorder As Color
		Public ShapeBackground1 As Color
		Public UseShapeBackground2 As Boolean
		Public ShapeBackground2 As Color
	End Class

End Class